#!/usr/bin/env python3

"""About module for kumaone"""

# Import builtin python libraries
from datetime import datetime

year = datetime.now().year

__version__ = "0.0.1a1"
__author__ = "Dalwar Hossain"
__license__ = "BSD 3-Clause"
__home_page__ = "https://kumaone.rtfd.io"
__copyright__ = f"{year}, {__author__}"
