#!/usr/bin/env python3

"""SocketIO events module for kumaone"""

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"


api_key_list = "apiKeyList"
auto_login = "autoLogin"
avg_ping = "avgPing"
cert_info = "certInfo"
connect = "connect"
disconnect = "disconnect"
docker_host_list = "dockerHostList"
heartbeat = "heartbeat"
heartbeat_list = "heartbeatList"
important_heartbeat_list = "importantHeartbeatList"
info = "info"
init_server_timezone = "initServerTimezone"
maintenance_list = "maintenanceList"
monitor_list = "monitorList"
notification_list = "notificationList"
proxy_list = "proxyList"
status_page_list = "statusPageList"
uptime = "uptime"
